#!/bin/sh
# Slackware Installation Script for SpamAssassin Non Required Modules
#Digest-HMAC-1.01
#Geography-Countries-1.4
#IP-Country-2.17
#Mail-SPF-Query-1.997
#Net-CIDR-Lite-0.15
#Net-DNS-0.48
#Sys-Hostname-Long-1.2
#URI-1.34

# Peter Hyman
# SA Non Required .pod file has to be concatenated onto existing
# perllocal.pod file if it exists.
 
cat /usr/lib/perl5/5.8.7/i486-linux/perllocal.pod.sanonreq >> /usr/lib/perl5/5.8.7/i486-linux/perllocal.pod

# remove temporary file.
rm /usr/lib/perl5/5.8.7/i486-linux/perllocal.pod.sanonreq
echo "SpamAssassin 3.0's Non Required modules have been installed."
echo "These modules will provide enhanced functionality to SpamAssassin!"
echo "If you have not done so already, Install the SpamAssassin program."
echo "More information is available at http://www.spamassassin.org and the"
echo "man pages."